#define addrLabelLength			16
#define addrNumFields			19
#define numPhoneFields			(phone5 - phone1 + 1)
#define numPhoneLabelsStoredSecond	(8 - numPhoneFields)

typedef struct {
	unsigned reserved		:7;
	unsigned sortByCompany	:1;
} AddrDBMisc;

typedef enum {
	name,
	firstName,
	company,
	phone1,
	phone2,
	phone3,
	phone4,
	phone5,
	address,
	city,
	state,
	zipCode,
	country,
	title,
	custom1,
	custom2,
	custom3,
	custom4,
	note,			// This field is assumed to be < 4K 
	addressFieldsCount
} AddressFields;

typedef union {
	struct {
		unsigned reserved:8;
		unsigned displayPhoneForList:4;	// The phone displayed for the list view 0 - 4
		unsigned phone5:4;				// Which phone (home, work, car, ...)
		unsigned phone4:4;
		unsigned phone3:4;
		unsigned phone2:4;
		unsigned phone1:4;
	} phones;
	UInt32 phoneBits;
} AddrOptionsType;

typedef union {
	struct {
		unsigned reserved	 :13;
		unsigned note        :1;	// set if record contains a note handle
		unsigned custom4     :1;	// set if record contains a custom4
		unsigned custom3     :1;	// set if record contains a custom3
		unsigned custom2     :1;	// set if record contains a custom2
		unsigned custom1     :1;	// set if record contains a custom1
		unsigned title       :1;	// set if record contains a title
		unsigned country	 :1;	// set if record contains a birthday
		unsigned zipCode	 :1;	// set if record contains a birthday
		unsigned state		 :1;	// set if record contains a birthday
		unsigned city		 :1;	// set if record contains a birthday
		unsigned address     :1;	// set if record contains a address
		unsigned phone5      :1;	// set if record contains a phone5
		unsigned phone4      :1;	// set if record contains a phone4
		unsigned phone3      :1;	// set if record contains a phone3
		unsigned phone2      :1;	// set if record contains a phone2
		unsigned phone1      :1;	// set if record contains a phone1
		unsigned company     :1;	// set if record contains a company
		unsigned firstName   :1;	// set if record contains a firstName
		unsigned name        :1;	// set if record contains a name (bit 0)
		
	} bits;
	UInt32 allBits;
} AddrDBRecordFlags;

typedef struct {
	AddrOptionsType	options;        // Display by company or by name
	Char *		fields[addressFieldsCount];
} AddrDBRecordType;

typedef AddrDBRecordType * AddrDBRecordPtr;

typedef struct {
      AddrOptionsType      options;        // Display by company or by name
      AddrDBRecordFlags      flags;
      unsigned char         companyFieldOffset;   // Offset from firstField
      char                  firstField;
} AddrPackedDBRecord;

typedef union {
	struct {
		unsigned reserved	:10;
		unsigned phone8		:1;	// set if phone8 label is dirty
		unsigned phone7		:1;	// set if phone7 label is dirty
		unsigned phone6     :1;	// set if phone6 label is dirty
		unsigned note       :1;	// set if note label is dirty
		unsigned custom4    :1;	// set if custom4 label is dirty
		unsigned custom3    :1;	// set if custom3 label is dirty
		unsigned custom2    :1;	// set if custom2 label is dirty
		unsigned custom1    :1;	// set if custom1 label is dirty
		unsigned title      :1;	// set if title label is dirty
		unsigned country	:1;	// set if country label is dirty
		unsigned zipCode	:1;	// set if zipCode label is dirty
		unsigned state		:1;	// set if state label is dirty
		unsigned city		:1;	// set if city label is dirty
		unsigned address    :1;	// set if address label is dirty
		unsigned phone5     :1;	// set if phone5 label is dirty
		unsigned phone4     :1;	// set if phone4 label is dirty
		unsigned phone3     :1;	// set if phone3 label is dirty
		unsigned phone2     :1;	// set if phone2 label is dirty
		unsigned phone1     :1;	// set if phone1 label is dirty
		unsigned company    :1;	// set if company label is dirty
		unsigned firstName  :1;	// set if firstName label is dirty
		unsigned name       :1;	// set if name label is dirty (bit 0)
	} bits;
	UInt32 allBits;
} AddrDBFieldLabelsDirtyFlags;

typedef char addressLabel[addrLabelLength];

typedef struct {
	UInt16 renamedCategories;	// bitfield of categories with a different name
	char categoryLabels[dmRecNumCategories][dmCategoryLength];
	UInt8 categoryUniqIDs[dmRecNumCategories];
	UInt8 lastUniqID;	// Uniq IDs generated by the device are between
						// 0 - 127.  Those from the PC are 128 - 255.
	UInt8 reserved1;	// from the compiler word aligning things
	UInt16 reserved2;
	AddrDBFieldLabelsDirtyFlags dirtyFieldLabels;
	addressLabel fieldLabels[addrNumFields + numPhoneLabelsStoredSecond];
	CountryType country;	// Country the database (labels) is formatted for
	UInt8 reserved;
	AddrDBMisc misc;
} AddrAppInfoType;

typedef AddrAppInfoType *AddrAppInfoPtr;
