/******************************************************************************
 *
 * Copyright (c) 1999 Palm Computing, Inc. or its subsidiaries.
 * All rights reserved.
 *
 * File: Starter.c
 *
 *****************************************************************************/

#include <PalmOS.h>
//#include <Find.h>
#include "AddrDB.h"
#include "HanTelRsc.h"


/***********************************************************************
 *
 *   Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
typedef struct 
{
	UInt16 tabLoc; // using lower 4bit
	UInt16 reserved;
	UInt16 category;
} HanTelPrefType;

#define TAB_WIDTH	26
#define TAB_NUMBER	5

#define GetPhoneLabel(r, p)	((r.options.phoneBits >> (((p) - phone1) << 2)) & 0xF)

/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/
//static Boolean HideSecretRecords;
static UInt16		CurrentCategory;
static BOOL			SavePos;		// TRUE is save CurTab else not.
static UInt16		CurTab=0;
static UInt16		DisplayType;	// 0:Last, 1:First, 2:FirstLast 3:LastFirst
static Char			DisplayS[2];	// ' ' or ',' or NONE(NULL)
static UInt32		RomVersion;
static Int16		TopVisibleRecord = 0;
static DmOpenRef	AddrDB;
static FontID		AddrListFont = stdFont;
static const UInt16	TabPart[7] =
 { 0x0000, 0xB0A1, 0xB6F3, 0xBBE7, 0xC2F7, 0xF000, 0x0000};
static MemHandle	RscBitmapH[TAB_NUMBER];
static BitmapPtr	RscBitmapPtr[TAB_NUMBER];
static Char PhoneLabelLetters[8];
static Char *PhoneListNum[8];
static Char	CategoryName[dmCategoryLength];

/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'HTeB'
#define appVersionNum			0x01
#define appPrefID				0x00
#define appPrefVersionNum		0x01
#define addrDBType				'DATA'
// Define the minimum OS version we support (2.0 for now).
#define ourMinVersion	sysMakeROMVersion(2,0,0,sysROMStageRelease,0)


/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32	romVersion;
	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);

	if (romVersion < requiredVersion)
	{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
							(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
		{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < ourMinVersion)
			{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		}
		
		return sysErrRomIncompatible;
	}
	//RomVersion = romVersion;
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}

// Address list table columns
#define nameAndNumColumn               	0
#define noteColumn                     	1

#define BitAtPosition(pos)                ((UInt32)1 << (pos))
#define GetBitMacro(bitfield, index)      ((bitfield) & BitAtPosition(index))

static Err AddrGetRecord(DmOpenRef dbP, UInt32 index,
			 AddrDBRecordPtr recordP, MemHandle *recordH);

static void AddrUnpack(AddrPackedDBRecord *src, AddrDBRecordPtr dest)
{
	UInt16 index;
	UInt32 flags;
	char *p;

	dest->options = src->options;
	flags = src->flags.allBits;
	p = &src->firstField;

	for(index=0;index<addressFieldsCount;index++)
	{
		// If the flag is set point to the string else NULL
		if (GetBitMacro(flags, index) != 0)
		{
			dest->fields[index] = p;
			p += StrLen(p) + 1;
		}
		else
			dest->fields[index] = NULL;
	}
}

static Boolean SeekRecord (UInt16 * indexP, Int16 offset, Int16 direction)
{
	Boolean result;
	UInt8 *namePtr;
	UInt16 firstChar;
	MemHandle recordH;
	AddrPackedDBRecord *r;

	DmSeekRecordInCategory (AddrDB, indexP, offset, direction, CurrentCategory);
	if (DmGetLastErr())
		return (false);

	recordH = DmQueryRecord(AddrDB, *indexP);
	r = (AddrPackedDBRecord *) MemHandleLock(recordH);
	ErrFatalDisplayIf(r == 0, "Addr Seek: data somehow missing");
	namePtr = (UInt8*)&r->firstField;
	firstChar = (namePtr[0]<<8) + namePtr[1];
	if(firstChar >= TabPart[CurTab] && firstChar <TabPart[CurTab+1])
		result = true;
	else
		result = false;

	MemHandleUnlock(recordH);

	return result;
}


static AddrAppInfoPtr AddrAppInfoGetPtr(DmOpenRef dbP)
{
	UInt16 cardNo;
	LocalID dbID;
	LocalID appInfoID;
   
	if (DmOpenDatabaseInfo(dbP, &dbID, NULL, NULL, &cardNo, NULL))
		return NULL;
	if (DmDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &appInfoID, NULL, NULL, NULL))
		return NULL;

	if (appInfoID == NULL) return NULL;
	return MemLocalIDToLockedPtr(appInfoID, cardNo);
}

static void CallAddrApp()
{
	LocalID dbID;
	Int16 row, col;
	UInt16 cardNo;
	DmSearchStateType dummy;
	GoToParamsPtr paramBlock;

	if(TblGetSelection(GetObjectPtr(MainListViewTable), &row, &col)==false)
		return;

	paramBlock = MemPtrNew(sizeof(GoToParamsType));

	// Gather data to call the Addressbook at record zero
	DmGetNextDatabaseByTypeCreator(true, &dummy, 'DATA', 'addr',
						true, &paramBlock->dbCardNo, &paramBlock->dbID);
	paramBlock->searchStrLen = 0;
	paramBlock->matchPos = 0;
	paramBlock->matchFieldNum = 0;
	paramBlock->matchCustom = 0;
	paramBlock->recordNum = TblGetRowID(GetObjectPtr(MainListViewTable), row);

	MemPtrSetOwner(paramBlock, 0);
	DmGetNextDatabaseByTypeCreator(true, &dummy, 'appl', 'addr', true, &cardNo, &dbID);
	SysUIAppSwitch(cardNo, dbID, sysAppLaunchCmdGoTo, paramBlock);

}

static void DrawNameTab (void)
{
	UInt16 i;
	UInt16 pos = 3;
	WinDrawLine(0, 31, 159, 31);
	
	for(i=0;i<TAB_WIDTH*5;i+=TAB_WIDTH)
	{
		WinDrawLine( i, 20, i, 31);
		WinDrawLine( i+2, 18, i+TAB_WIDTH-2, 18);
		WinDrawLine( i+2, 18, i, 20);
		WinDrawLine( i+TAB_WIDTH-2, 18, i+TAB_WIDTH, 20);

	}
	WinDrawLine( i, 20, i, 31);
	WinEraseLine(1+TAB_WIDTH*CurTab, 31, TAB_WIDTH-1+TAB_WIDTH*CurTab, 31);

	for(i=0;i<TAB_NUMBER;i++)
	{
		WinDrawBitmap(RscBitmapPtr[i], 2+TAB_WIDTH*i, 20);
	}
}

static UInt16 GetVisibleRows(TablePtr table)
{
	UInt16 rows;
	FontID currFont;
	RectangleType rect;

	TblGetBounds( table, &rect);
	currFont = FntSetFont(AddrListFont);
	rows = rect.extent.y / FntLineHeight();
	FntSetFont(currFont);

	return rows;
}

static void PopupPhoneList(EventPtr eventP)
{
	UInt16 i, j;
	UInt16 maxLeng;
	UInt16 totalLeng;
	UInt16 curRecord;
	UInt32 phoneLabel;
	MemHandle recordH;
	AddrDBRecordType record;
	ListPtr popupPhoneList;
	MemHandle hPhoneNum;
	Char *pPhoneNum;

	curRecord = TblGetRowID(eventP->data.tblSelect.pTable,
						eventP->data.tblSelect.row);

	AddrGetRecord(AddrDB, curRecord, &record, &recordH);

	maxLeng = totalLeng = 0;
	for(i=phone1;i<phone1+5;i++)
	{
		if(record.fields[i])
		{
			totalLeng += StrLen(record.fields[i]);
			maxLeng = max( maxLeng,
					FntCharsWidth( record.fields[i], StrLen(record.fields[i])));
		}
	}
	maxLeng = min( 140, maxLeng);

	hPhoneNum = MemHandleNew(totalLeng+3*5);
	pPhoneNum = MemHandleLock(hPhoneNum);
	phoneLabel = record.options.phoneBits;
	for(i=phone1,j=0;i<phone1+5;i++)
	{
		if(record.fields[i])
		{
			PhoneListNum[j] = pPhoneNum;
			*pPhoneNum ++ = PhoneLabelLetters[phoneLabel&0x0F];
			*pPhoneNum ++ = ':';
			StrCopy( pPhoneNum, record.fields[i]);
			pPhoneNum += StrLen(record.fields[i]);
			*pPhoneNum ++ = NULL;
			j++;
		}
		phoneLabel >>= 4;
	}

	popupPhoneList = GetObjectPtr (MainPhoneListList);
	popupPhoneList->bounds.extent.x = maxLeng+17;
	LstSetListChoices(popupPhoneList, PhoneListNum, j);
	LstSetHeight( popupPhoneList, j);
	LstSetPosition( popupPhoneList, 40, eventP->data.tblSelect.row*11+37);
	LstSetSelection( popupPhoneList, -1);
	LstPopupList( popupPhoneList );

	MemHandleUnlock( recordH );

	MemHandleUnlock( hPhoneNum );
	MemHandleFree( hPhoneNum );
}

static void PopupPhoneNote(EventPtr eventP)
{
	Err error;
	UInt16 curRecord;
	MemHandle recordH;
	AddrDBRecordType record;
	FormPtr noteForm;
	FieldPtr fldPtr;

	curRecord = TblGetRowID(eventP->data.tblSelect.pTable,
						eventP->data.tblSelect.row);

	error = AddrGetRecord(AddrDB, curRecord, &record, &recordH);
	ErrNonFatalDisplayIf ((error), "Record not found");

	if(record.fields[note])
	{
		noteForm = FrmInitForm(NoteForm);
		FrmSetTitle( noteForm, record.fields[name]);
		fldPtr = FrmGetObjectPtr( noteForm, FrmGetObjectIndex( noteForm, NoteNoteField));
		FldSetTextPtr( fldPtr, record.fields[note]);
		FldRecalculateField( fldPtr, true);
		FrmDoDialog(noteForm);
		FldSetTextPtr( fldPtr, NULL);
		FrmDeleteForm(noteForm);
	}
	MemHandleUnlock(recordH);
}

/************************************************************
 *
 *  FUNCTION: AddrGetRecord
 *
 *  DESCRIPTION: Get a record from the Address Database
 *
 *  PARAMETERS: database pointer - open db pointer
 *            database index - index of record to lock
 *            address record pointer - pointer address structure
 *            address record - handle to unlock when done
 *
 *  RETURNS: ##0 if successful, errorcode if not
 *    The record's handle is locked so that the pointer to 
 *  strings within the record remain pointing to valid chunk
 *  versus the record randomly moving.  Unlock the handle when
 *  AddrDBRecord is destroyed.
 *
 *  CREATED: 1/14/95 
 *
 *  BY: Roger Flores
 *
 *************************************************************/

static Err AddrGetRecord(DmOpenRef dbP, UInt32 index, AddrDBRecordPtr recordP, 
					MemHandle *recordH)
{
	AddrPackedDBRecord *src;

	*recordH = DmQueryRecord(dbP, index);
	src = (AddrPackedDBRecord *) MemHandleLock(*recordH);
	if (src == NULL)
		return dmErrIndexOutOfRange;
   
	AddrUnpack(src, recordP);
	
	return 0;
}


static Boolean SearchCategory(DmOpenRef dbP, UInt16 *indexP)
{
	Int16 numOfRecords;
	MemHandle rH;
	AddrPackedDBRecord* r;
	UInt16 kmin, probe, i;		// all positions in the database.
	Int16 result;				// result of comparing two records
	UInt8 *pName;
	UInt16 firstCh;

	numOfRecords = DmNumRecords(dbP);
	if(numOfRecords == 0)
		return false;

	result = 0;
	kmin = probe = 0;
	rH = 0;

	while (numOfRecords > 0)
	{
		i = numOfRecords / 2;
		probe = kmin + i;

		// Compare the two records.  Treat deleted records as greater.
		// If the records are equal look at the following position.
		rH = DmQueryRecord(dbP, probe);
		if (rH == 0)
		{
			result = -1;      // Delete record is greater
		}
		else
		{
			r = (AddrPackedDBRecord *) MemHandleLock(rH);
			ErrFatalDisplayIf(r == 0, "Addr bsearch: data somehow missing");

			// Compare the string to the first sort key only
			pName = (UInt8*)&r->firstField;
			firstCh = (pName[0]<<8) + pName[1];
			if(TabPart[CurTab] <= firstCh)
			{
				if(TabPart[CurTab+1] > firstCh)
					result = 0;
				else
					result = -1;
			}
			else
				result = 1;
			MemHandleUnlock(rH);
		}

		// More likely than < 0 because of deleted records
		if (result < 0)
			numOfRecords = i;
		else if(result > 0)
		{
			kmin = probe + 1;
			numOfRecords = numOfRecords - i - 1;
		}
		else
		{
			// I found wanted record
			while (probe)
			{
				rH = DmQueryRecord (dbP, probe-1);
				if(rH == 0)
				{
					result = -1;
				}
				else
				{
					r = (AddrPackedDBRecord *) MemHandleLock(rH);
					ErrFatalDisplayIf(r == 0, "Addr bsearch: data somehow missing");
					pName = (UInt8*)&r->firstField;
					firstCh = (pName[0]<<8) + pName[1];
					if(TabPart[CurTab] <= firstCh)
						result = -1;
					else
						result = 0;
					MemHandleUnlock(rH);
				}

				if(result == 0)
				{
					*indexP = probe;
					probe = 0;
				}
				else
				{
					probe --;
					if(probe == 0) *indexP = 0;
				}
			}
			break;
		}
	}

	return true;
}


/***********************************************************************
 *
 * FUNCTION:    ListViewUpdateScrollButtons
 *
 * DESCRIPTION: Show or hide the list view scroll buttons.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *         Name   Date      Description
 *         ----   ----      -----------
 *         roger   6/21/95   Initial Revision
 *
 ***********************************************************************/
static void ListViewUpdateScrollButtons (void)
{
	UInt16	row;
	UInt16	upIndex;
	UInt16	downIndex;
	UInt16	recordNum;
	Boolean	scrollableUp;
	Boolean	scrollableDown;
	FormPtr	frm;
	TablePtr table;

	frm = FrmGetActiveForm ();

	// Update the button that scroll the list.
	//
	// If the first record displayed is not the fist record in the category,
	// enable the up scroller.
	recordNum = TopVisibleRecord;
	scrollableUp = SeekRecord(&recordNum, 1, dmSeekBackward);

	// Find the record in the last row of the table
	table = GetObjectPtr (MainListViewTable);
	row = TblGetLastUsableRow (table);
	if(row != tblUnusableRow)
		recordNum = TblGetRowID(table, row);

	// If the last record displayed is not the last record in the category,
	// enable the down scroller.
	scrollableDown = SeekRecord(&recordNum, 1, dmSeekForward);

	// Update the scroll button.
	upIndex = FrmGetObjectIndex (frm, MainScrollUpRepeating);
	downIndex = FrmGetObjectIndex (frm, MainScrollDownRepeating);
	FrmUpdateScrollers (frm, upIndex, downIndex, scrollableUp, scrollableDown);
}


/***********************************************************************
 *
 * FUNCTION:    ListLoadTable
 *
 * DESCRIPTION: This routine loads address book database records into
 *              the list view form.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *         Name   Date      Description
 *         ----   ----      -----------
 *         art   6/5/95      Initial Revision
 *
 ***********************************************************************/
static void ListLoadTable (void)
{
	UInt16	row;
	UInt16	numRows;
	UInt16	lineHeight;
	UInt16	visibleRows;
	UInt16	recordNum;
	UInt16	totalRecord;
	FontID	currFont;
	TablePtr table;
	RectangleType rect;

	// For each row in the table, store the record number as the row id.
	table = GetObjectPtr (MainListViewTable);
	TblGetBounds(table, &rect);

	TblUnhighlightSelection(table);

	currFont = FntSetFont (AddrListFont);
	lineHeight = FntLineHeight();
	FntSetFont (currFont);

	visibleRows = rect.extent.y / lineHeight;
	numRows = TblGetNumberOfRows (table);
	if(visibleRows > numRows)
		visibleRows = numRows;

	recordNum = TopVisibleRecord;
	totalRecord = DmNumRecords(AddrDB);

	for(row=0;row<visibleRows;row++)
	{
		if(!SeekRecord(&recordNum, 0, dmSeekForward))
			break;

		// Make the row usable.
		TblSetRowUsable(table, row, true);
      
		//DmRecordInfo (AddrDB, recordNum, &attr, NULL, NULL);
		//masked = (((attr & dmRecAttrSecret) && PrivateRecordVisualStatus == maskPrivateRecords));
		//TblSetRowMasked(table,row,masked);

		TblMarkRowInvalid(table, row);

		TblSetRowID(table, row, recordNum);

		TblSetItemFont (table, row, nameAndNumColumn, AddrListFont);
		TblSetRowHeight (table, row, lineHeight);

		recordNum++;
	}
   

	// Hide the item that don't have any data.
	while (row < numRows)
	{      
		TblSetRowUsable (table, row, false);
		row++;
	}

	ListViewUpdateScrollButtons();
}


/***********************************************************************
 *
 * FUNCTION:    ListViewScroll
 *
 * DESCRIPTION: This routine scrolls the list of names and phone numbers 
 *              in the direction specified.
 *
 * PARAMETERS:  direction	- up or dowm
 *              units		- unit amount to scroll
 *              byLine		- if true, list scrolls in line units
 *									- if false, list scrolls in page units
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *			Name		Date		Description
 *			----		----		-----------
 *			art		6/5/95	Initial Revision
 *       frigino	8/14/97	Modified to scroll by line or page in units
 *       gap   	10/12/99	Close command bar before processing scroll
 *       gap   	10/15/99	Clean up selection handling after scroll
 *       gap   	10/25/99	Optimized scrolling to only redraw if item position changed
 *
 ***********************************************************************/
static void ListViewScroll (WinDirectionType direction, UInt16 units, Boolean byLine)
{
	TablePtr table;
	UInt16 rowsInPage;
	UInt16 visibleRows;
	UInt16 newTopVisibleRecord;
	UInt16 prevTopVisibleRecord = TopVisibleRecord;
	
	
	// Before processing the scroll, be sure that the command bar has been closed.
	MenuEraseStatus (0);

	table = GetObjectPtr (MainListViewTable);
	// Safe. There must be at least one row in the table.
	visibleRows = GetVisibleRows(table);
	rowsInPage = TblGetNumberOfRows(table);
	rowsInPage = min(visibleRows, rowsInPage);
	rowsInPage --;
	newTopVisibleRecord = TopVisibleRecord;

	// Scroll the table down.
	if (direction == winDown)
	{
		// Scroll down by line units
		if (byLine)
		{
			// Scroll down by the requested number of lines
			if (!SeekRecord (&newTopVisibleRecord, units, dmSeekForward))
			{
				// Tried to scroll past bottom. Goto last record
				newTopVisibleRecord = dmMaxRecordIndex;
				SeekRecord (&newTopVisibleRecord, 1, dmSeekBackward);
			}
		}
		else // Scroll in page units
		{
			// Try scrolling down by the requested number of pages
			if (!SeekRecord (&newTopVisibleRecord, units * rowsInPage, dmSeekForward))
			{
				return;
				/*// Hit bottom. Try going backwards one page from the last record
				newTopVisibleRecord = dmMaxRecordIndex;
				if (!SeekRecord (&newTopVisibleRecord, rowsInPage, dmSeekBackward))
				{
					// Not enough records to fill one page. Goto the first record
					newTopVisibleRecord = 0;
					SeekRecord (&newTopVisibleRecord, 0, dmSeekForward);
				}*/
			}
		}
	}
	else // Scroll the table up
	{
		// Scroll up by line units
		if (byLine)
		{
			// Scroll up by the requested number of lines
			if (!SeekRecord (&newTopVisibleRecord, units, dmSeekBackward))
			{
				// Tried to scroll past top. Goto first record
				newTopVisibleRecord = 0;
				SeekRecord (&newTopVisibleRecord, 0, dmSeekForward);
			}
		}
		// Scroll in page units
		else
		{
			// Try scrolling up by the requested number of pages
			if (!SeekRecord (&newTopVisibleRecord, units * rowsInPage, dmSeekBackward))
			{
				return;
				// Hit top. Goto the first record
				//newTopVisibleRecord = 0;
				//SeekRecord (&newTopVisibleRecord, 0, dmSeekForward);
			}
		}
	}

	// Avoid redraw if no change
	if (TopVisibleRecord != newTopVisibleRecord)
	{
		TopVisibleRecord = newTopVisibleRecord;
		//CurrentRecord = noRecord;  	// scrolling always deselects current selection
		ListLoadTable();
		
		// Need to compare the previous top record to the current after ListLoadTable 
		// as it will adjust TopVisibleRecord if drawing from newTopVisibleRecord will
		// not fill the whole screen with items.
		if (TopVisibleRecord != prevTopVisibleRecord)
			TblRedrawTable(table);
	}
}


/***********************************************************************
 *
 * FUNCTION:    ListViewDrawRecord
 *
 * DESCRIPTION: This routine draws an address book record.  It is called as
 *              a callback routine by the table object.
 *
 * PARAMETERS:  table  - pointer to the address list table
 *              row    - row number, in the table, of the item to draw
 *              column - column number, in the table, of the item to draw
 *              bounds - bounds of the draw region
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *         Name   Date      Description
 *         ----   ----      -----------
 *         roger   6/21/95   Initial Revision
 *
 ***********************************************************************/
const Int16 PhoneColumnWidth = 82;
const Int16 spaceBetweenNamesAndPhoneNumbers = 6;
static void ListViewDrawRecord(MemPtr table, Int16 row, Int16 column, 
								RectanglePtr bounds)
{
	Err error;
	UInt16 recordNum;
	UInt16 phoneLabel;
	AddrDBRecordType record;
	MemHandle recordH;
	Char noteChar;
	Char * phone;
	Char * pName;
	Int16 phoneLength, nameLength;
	Int16 phoneWidth;//, nameWidth;
	Boolean ignored;
	FontID currFont;
	Char *pMergeNames = NULL;

	WinEraseRectangle(bounds, 0);
	// Get the record number that corresponds to the table item to draw.
	// The record number is stored in the "intValue" field of the item.
	recordNum = TblGetRowID(table, row);

	error = AddrGetRecord(AddrDB, recordNum, &record, &recordH);
	ErrNonFatalDisplayIf ((error), "Record not found");
	if(error) return;

	if(column == 0)
	{
		currFont = FntSetFont(AddrListFont);

		bounds->extent.x -= FntCharWidth('W');

		if((DisplayType&0x0F)>1)
		{
			nameLength = 0;
			if(record.fields[name])
				nameLength += StrLen( record.fields[name]);
			if(record.fields[firstName])
				nameLength += StrLen( record.fields[firstName]);
			nameLength += 3;
			pMergeNames = MemPtrNew(nameLength);
			pMergeNames[0] = NULL;
			pMergeNames[1] = NULL;
			pName = pMergeNames;
		}
		switch(DisplayType&0x0F)
		{
			case 0 : // Only last
				pName = record.fields[name] ? record.fields[name] : record.fields[firstName];
				break;
			case 1 : // Only first
				pName = record.fields[firstName] ? record.fields[firstName] : record.fields[name];
				break;
			case 2 : // first SEPARATOR last
				if(record.fields[firstName])
					StrCat( pMergeNames, record.fields[firstName]);
				if(record.fields[name])
				{
					StrCat( pMergeNames, DisplayS);
					StrCat( pMergeNames, record.fields[name]);
				}
				break;
			case 3 : // last SEPARATOR first
				if(record.fields[name])
					StrCat( pMergeNames, record.fields[name]);
				if(record.fields[firstName])
				{
					StrCat( pMergeNames, DisplayS);
					StrCat( pMergeNames, record.fields[firstName]);
				}
				break;
		}
		//nameWidth = bounds->extent.x;
		//nameLength = nameWidth;
		//FntCharsInWidth( pName, &nameWidth, &nameLength, &ignored);
		if(pName)
			nameLength = StrLen(pName);
		if(nameLength)
		{
			WinDrawChars( pName, nameLength,
						bounds->topLeft.x, bounds->topLeft.y);
		}
		else
		{
			nameLength = StrLen(record.fields[company]);
			if(nameLength)
				WinDrawChars( record.fields[company], nameLength,
							bounds->topLeft.x, bounds->topLeft.y);
			else
			{
				MemHandle hStr;
				hStr = DmGetResource(strRsc, UnnamedString);
				WinDrawChars( MemHandleLock(hStr), 9, bounds->topLeft.x, bounds->topLeft.y);
				MemHandleUnlock(hStr);
				DmReleaseResource(hStr);
			}
		}
		phone = record.fields[phone1 + record.options.phones.displayPhoneForList];
		if(phone)
		{
			phoneWidth = bounds->extent.x;
			phoneLength = phoneWidth;
			FntCharsInWidth (phone, &phoneWidth, &phoneLength, &ignored);
			WinDrawChars( phone, StrLen(phone),
				bounds->topLeft.x+bounds->extent.x-phoneWidth, bounds->topLeft.y);
			phoneLabel = GetPhoneLabel(record, phone1 + 
							record.options.phones.displayPhoneForList);
			phoneLength = (FntCharWidth('W') -
						FntCharWidth(PhoneLabelLetters[phoneLabel])+1)>>1;
			WinDrawChars(&PhoneLabelLetters[phoneLabel], 1,
					bounds->topLeft.x+bounds->extent.x+phoneLength, bounds->topLeft.y);
		}
		FntSetFont (currFont);
	}
	else
	{
		// Draw a note symbol if the field has a note
		if (record.fields[note])
		{
			currFont = FntSetFont (symbolFont);
			noteChar = symbolNote;
			WinDrawChars (&noteChar, 1, bounds->topLeft.x, bounds->topLeft.y);
			FntSetFont (currFont);
		}
	}
	MemHandleUnlock(recordH);

	if(pMergeNames)
		MemPtrFree(pMergeNames);
}


/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr frmP)
{
	UInt16 row;
	UInt16 rowsInTable;
	TablePtr table;
	ControlPtr ctl;

	// Initialize the address list table.
	table = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, MainListViewTable));
	rowsInTable = TblGetNumberOfRows(table);

	for(row=0;row<rowsInTable;row++)
	{      
		TblSetItemStyle(table, row, nameAndNumColumn, customTableItem);
		TblSetItemStyle(table, row, noteColumn, customTableItem);
		TblSetRowUsable(table, row, false);
	}

	TblSetColumnUsable(table, nameAndNumColumn, true);
	TblSetColumnUsable(table, noteColumn, true);


	// Set the callback routine that will draw the records.
	TblSetCustomDrawProcedure(table, nameAndNumColumn, ListViewDrawRecord);
	TblSetCustomDrawProcedure(table, noteColumn, ListViewDrawRecord);


	// Load records into the address list.
	ListLoadTable();

	DrawNameTab();

	ctl = GetObjectPtr (MainCategoryPopTrigger);
	CategoryGetName (AddrDB, CurrentCategory, CategoryName);
	CategorySetTriggerLabel(ctl, CategoryName);
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	FormPtr frmP;

	switch (command)
	{
		case MainOptionsPreferences:
			MenuEraseStatus(0);
			FrmPopupForm(PrefForm);
			handled = true;
			break;
		case MainOptionsAboutHanTel:
			MenuEraseStatus(0);					// Clear the menu status from the display.
			frmP = FrmInitForm (AboutForm);
			FrmDoDialog (frmP);					// Display the About Box.
			FrmDeleteForm (frmP);
			handled = true;
			break;
	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
	UInt16 startPart;
	UInt16 category;
	Boolean handled = false;
	Boolean categoryEdited;
	FormPtr frmP;

	static Int16 sScrnX;
	switch (eventP->eType) 
	{
		case tblSelectEvent :
			if(eventP->data.tblSelect.column == 0)
			{
				if(sScrnX < 50)
					CallAddrApp();
				else
					PopupPhoneList( eventP );
			}
			else
				PopupPhoneNote( eventP );
			break;

		case penDownEvent:
			sScrnX = eventP->screenX;
			if(eventP->screenY > 20 && eventP->screenY < 31)
			{
				if(eventP->screenX < TAB_WIDTH*5)
				{
					WinDrawLine(1+TAB_WIDTH*CurTab, 31,
						 TAB_WIDTH-1+TAB_WIDTH*CurTab, 31);
					CurTab = eventP->screenX / TAB_WIDTH;
					WinEraseLine(1+TAB_WIDTH*CurTab, 31,
						 TAB_WIDTH-1+TAB_WIDTH*CurTab, 31);
					if(CurTab == 5)
					{
						TopVisibleRecord = 0;
					}
					else
					{
						SearchCategory(AddrDB, &startPart);
						TopVisibleRecord = startPart;
					}
					ListLoadTable();
					FrmDrawForm(FrmGetActiveForm());
				}
			}
			break;

		 case ctlSelectEvent:
			if(eventP->data.ctlSelect.controlID == MainCategoryPopTrigger)
			{
				category = CurrentCategory;
				frmP = FrmGetActiveForm();
				categoryEdited = CategorySelect (AddrDB, frmP,
									MainCategoryPopTrigger, MainCategoryListList,
									true, &category, CategoryName, 1,
									categoryHideEditCategory);
				if(category != CurrentCategory)
				{
					CurrentCategory = category;
					ListLoadTable();
					FrmDrawForm(FrmGetActiveForm());
				}
				handled = true;
			}
			if(MainEditButton == eventP->data.ctlSelect.controlID)
			{
				CallAddrApp();
				handled = true;
			}
			break;
		case ctlRepeatEvent:
			switch (eventP->data.ctlRepeat.controlID)
			{
				case MainScrollUpRepeating:
					ListViewScroll (winUp, 1, false);
					break;
				case MainScrollDownRepeating:
					ListViewScroll (winDown, 1, false);
					break;
			}
			break;
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case keyDownEvent:
			if(EvtKeydownIsVirtual(eventP))
			{
				switch(eventP->data.keyDown.chr)
				{
					case vchrPageUp:
						ListViewScroll (winUp, 1, false);
						handled = true;
						break;
					case vchrPageDown:
						ListViewScroll (winDown, 1, false);
						handled = true;
						break;
				}
			}
			break;
		case frmOpenEvent:
			SearchCategory(AddrDB, &startPart);
			TopVisibleRecord = startPart;
			frmP = FrmGetActiveForm();
			MainFormInit(frmP);
			FrmDrawForm(frmP);
			handled = true;
			break;

		default:
			break;

	}
	return handled;
}

static Boolean PrefFormHandleEvent(EventPtr eventP)
{
	FormPtr frmP;
	Boolean handled=false;

	switch (eventP->eType) 
	{
		case ctlSelectEvent:
			switch(eventP->data.ctlSelect.controlID)
			{
				case PrefOkButton:
					SavePos = CtlGetValue(GetObjectPtr(PrefSavePosCheckbox));
					DisplayType = LstGetSelection(GetObjectPtr(PrefDisplayList)) |
									(LstGetSelection(GetObjectPtr(PrefSeparatorList))<<4);
					switch(DisplayType>>4)
					{
						//case 0 : DisplayS[0] = ' '; break;
						case 1 : DisplayS[0] = ','; break;
						case 2 : DisplayS[0] = NULL; break;
						default: DisplayS[0] = ' '; break;// 0 is same
					}

				case PrefCancelButton:
					frmP = FrmGetActiveForm();
					FrmEraseForm(frmP);
					FrmDeleteForm(frmP);
					FrmSetActiveForm(FrmGetFirstForm());
					FrmUpdateForm(FrmGetActiveFormID(), frmRedrawUpdateCode);
					handled = true;
					break;
			}
			break;
		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			CtlSetValue(GetObjectPtr(PrefSavePosCheckbox), SavePos);
			CtlSetLabel(GetObjectPtr(PrefDTypePopTrigger),
					LstGetSelectionText(GetObjectPtr(PrefDisplayList), DisplayType&0x0F));
			LstSetSelection(GetObjectPtr(PrefDisplayList), DisplayType&0x0F);
			CtlSetLabel(GetObjectPtr(PrefSTypePopTrigger),
					LstGetSelectionText(GetObjectPtr(PrefSeparatorList), DisplayType>>4));
			LstSetSelection(GetObjectPtr(PrefSeparatorList), DisplayType>>4);
			FrmDrawForm (frmP);
			handled = true;
			break;
		default:
			break;
	}
	return handled;
}
/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
	{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
		{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;
			case PrefForm:
				FrmSetEventHandler(frmP, PrefFormHandleEvent);
				break;
			default:
				ErrFatalDisplay("Invalid Form Load Event");
				break;

		}
		return true;
	}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}

/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	UInt8 i;
	Int16 prefsRet;
	UInt16 prefsSize;
	HanTelPrefType prefs;
	AddrAppInfoPtr appInfoPtr;

	prefsRet = PrefGetAppPreferences(appFileCreator, appPrefID,
								 	&prefs, &prefsSize, true);

	DisplayS[0] = NULL;
	DisplayS[1] = NULL;
	CurTab = 1; // 1 is ,, 
	SavePos = true;
	DisplayType = 0;
	CurrentCategory = dmAllCategories;

	if((prefsRet!=noPreferenceFound) && (prefsSize == sizeof(prefs)))
	{
		CurTab = prefs.tabLoc&0x0F;
		CurrentCategory = prefs.category;
		if(CurTab == 0x0F)
		{
			CurTab = 1;
			SavePos = false;
			CurrentCategory = dmAllCategories;
		}
		
		DisplayType = prefs.reserved;
		switch(DisplayType>>4)
		{
			case 0 : DisplayS[0] = ' '; break;
			case 1 : DisplayS[0] = ','; break;
			case 2 : DisplayS[0] = NULL; break;
			default: DisplayS[0] = ' '; break;
		}
	}

	// Find the application's data file.  If it doesn't exist create it.
	AddrDB = DmOpenDatabaseByTypeCreator(addrDBType, sysFileCAddress, dmModeReadOnly);
	if(!AddrDB)
		return 1;

	for(i=0;i<TAB_NUMBER;i++)
	{
		RscBitmapH[i] = DmGetResource( bitmapRsc, FirstTabBitmap+i);
		RscBitmapPtr[i] = MemHandleLock(RscBitmapH[i]);
	}

	appInfoPtr = (AddrAppInfoPtr) AddrAppInfoGetPtr(AddrDB);
	ErrFatalDisplayIf(appInfoPtr == NULL, "Missing app info block");

	for (i=0;i<8;i++)
	{
		PhoneLabelLetters[i] = appInfoPtr->fieldLabels[i + 
				((i < numPhoneFields) ? phone1 : 
				(addressFieldsCount - numPhoneFields))][0];
	}

	MemPtrUnlock(appInfoPtr);

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	UInt16 i;
	HanTelPrefType prefs;

	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.

	if(SavePos)
	{
		prefs.tabLoc = CurTab;
		prefs.category = CurrentCategory;
	}
	else
	{
		prefs.tabLoc = 0x0F;
		prefs.category = dmAllCategories;
	}
	prefs.reserved = DisplayType;

	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof(prefs), true);
		
	// Close all the open forms.
	FrmCloseAllForms ();

	DmCloseDatabase(AddrDB);
	for(i=0;i<TAB_NUMBER;i++)
	{
		MemPtrUnlock(RscBitmapPtr[i]);
		DmReleaseResource(RscBitmapH[i]);
	}
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
	{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;

		default:
			break;

	}
	
	return errNone;
}

